unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls,
  Contnrs,

  jpeg,
  pngimage,
  DateUtils,

  IniFiles,
  ExePathUnit,
  ShellUnit,
  StringUnit,
  EnumWinUnit,
  SystemUnit,
  CreateProcessUnit,
  RectPointUnit,
  WindowUnit,
uses_end;

type
  TSplashForm = class(TForm)
    Image1: TImage;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    FDelphiFullPath: String;
    FDelphiIDEProcessID: Integer;
    FTimerEventFlag: Boolean;
    FStartTime: TDateTime;
    FSplashFileName: String;
    procedure IniRead;
    procedure IniWrite;

  public
    { Public declarations }
  end;

var
  SplashForm: TSplashForm;

implementation

{$R *.dfm}


procedure SetProcessPriority(ProcessID, Priority: DWORD);
var
  ProcessHandle: THandle;
begin
  ProcessHandle := OpenProcess(PROCESS_SET_INFORMATION, false, ProcessID);
  SetPriorityClass(ProcessHandle, Priority);
end;

procedure TSplashForm.FormCreate(Sender: TObject);
var
  SplashFilePath: string;
begin
  SetPriorityClass(GetCurrentProcess, IDLE_PRIORITY_CLASS);

  FTimerEventFlag := False;
  Timer1.Enabled := False;
  Timer1.Interval := 500;
  FStartTime := Now; //X^[g

  Self.BorderStyle := bsNone;
//  Self.Position := poScreenCenter;
//  Self.FormStyle := fsStayOnTop;

  IniRead;

  SplashFilePath := AppFolderPath+PathDelim+'splash'+PathDelim+FSplashFileName;
  if FileExists(SplashFilePath) then
  begin
    Image1.Picture.LoadFromFile(SplashFilePath);
    Self.Width := Image1.Picture.Width;
    Self.Height := Image1.Picture.Height;
  end else
  begin
    Self.Width := 0;
    Self.Height := 0;
  end;
  Self.BoundsRect := RectMoveCenter(Self.BoundsRect,
    RectCenter(NoneTaskBarDesktopRect));

  if FileExists(FDelphiFullPath) then
  begin
    FDelphiIDEProcessID :=
      EasyCreateProcessCmd(IncludeBothEndsStr(FDelphiFullPath, '"')+' /ns', False);
    Timer1.Enabled := True;
  end else
  begin
    Application.Terminate;
  end;

  IniWrite;
end;


procedure TSplashForm.IniRead;
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(AppIniFilePath);
  with Ini do try
  FDelphiFullPath := ReadString('Delphi', 'FullPath', '');
  FSplashFileName := ReadString('Splash', 'FileName', '');
  finally Ini.Free; end;
end;

procedure TSplashForm.IniWrite;
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(AppIniFilePath);
  with Ini do try
  WriteString('Delphi', 'FullPath', FDelphiFullPath);
  WriteString('Splash', 'FileName', FSplashFileName);
  finally Ini.UpdateFile; Ini.Free; end;
end;


procedure TSplashForm.Timer1Timer(Sender: TObject);
var
  WinList: TObjectList;
  WinInfo: TWinInfo;
  i: Integer;
begin
  if 3 <= MinutesBetween(Now, FStartTime) then Self.Close;

  if FTimerEventFlag then Exit;

  FTimerEventFlag := True; try
  WinList := TObjectList.Create; try
  EnumWindowInfo(WinList, 1, False);
  for i := 0 to WinList.Count - 1 do
  begin
    WinInfo := TWinInfo(WinList.Items[i]);
    if WinInfo.ProcessID <> FDelphiIDEProcessID then Continue;

    if WinInfo.Visible = False then Continue;
    if SameText(WinInfo.WindowClass, 'TApplication') then Continue;
    Self.Close;

  end;
  finally WinList.Free; end;
  finally FTimerEventFlag := False; end;
end;

end.
